using System;
using System.Collections.Generic;
using System.Text;
using NServiceBus.Unicast.Transport;
using Utils;
using NServiceBus.Messages;
using NServiceBus.Unicast.Transport.Msmq;
using log4net;
using System.Threading;

namespace NServiceBus.Unicast.MsmqDistributor
{
    public class Distributor
    {
        #region config info

        public string SubscriptionInputQueue
        {
            set { this.subscriptionInfoTransport.InputQueue = value; }
        }

        public bool UseXmlSerializationForSubscriptions
        {
            set { this.subscriptionInfoTransport.UseXmlSerialization = value; }
        }

        public string MessageBusInputQueue
        {
            set { this.messageBusTransport.InputQueue = value; }
        }

        public bool UseXmlSerializationForMessageBus
        {
            set { this.messageBusTransport.UseXmlSerialization = value; }
        }

        public string ErrorQueue
        {
            set
            {
                this.subscriptionInfoTransport.ErrorQueue = value;
                this.messageBusTransport.ErrorQueue = value;
            }
        }

        public string MsmqStorageQueue
        {
            set { this.subscriberManager = new MsmqSubscriberManager(value); }
        }

        #endregion

        #region public methods

        public void Start()
        {
            this.messageBusTransport.SkipDeserialization = true;
            this.messageBusTransport.IsTransactional = true;
            this.messageBusTransport.TimeToWaitForReceive = "0:00:10";
            this.messageBusTransport.NumberOfWorkerThreads = 1;

            this.subscriptionInfoTransport.IsTransactional = true;
            this.subscriptionInfoTransport.TimeToWaitForReceive = "0:00:10";
            this.subscriptionInfoTransport.NumberOfWorkerThreads = 1;
            this.subscriptionInfoTransport.MessageTypesToBeReceived = new List<Type>(new Type[] { typeof(ReadyMessage) });

            this.messageBusTransport.MsgReceived += new EventHandler<MsgReceivedEventArgs>(messageBusTransport_MsgReceived);
            this.subscriptionInfoTransport.MsgReceived += new EventHandler<MsgReceivedEventArgs>(subscriptionInfoTransport_MsgReceived);

            this.subscriptionInfoTransport.Start();
            this.messageBusTransport.Start();
        }

        public void Stop()
        {
            this.messageBusTransport.Dispose();
            this.subscriptionInfoTransport.Dispose();
        }

        #endregion

        #region helper methods

        void subscriptionInfoTransport_MsgReceived(object sender, MsgReceivedEventArgs e)
        {
            if (e.Message.Body != null)
                if (e.Message.Body.Length == 1)
                    if (e.Message.Body[0] is ReadyMessage)
                    {
                        logger.Debug("Server available: " + e.Message.ReturnAddress);
                        this.subscriberManager.AddSubscriber(e.Message.ReturnAddress);
                    }
        }

        void messageBusTransport_MsgReceived(object sender, MsgReceivedEventArgs e)
        {
            string destination = this.subscriberManager.PopSubscriber();

            if (destination == null)
                this.messageBusTransport.ReceiveMessageLater(e.Message);
            else
            {
                logger.Debug("Sending message to: " + destination);
                this.messageBusTransport.Send(e.Message, destination);
            }
        }

        #endregion

        #region members

        private MsmqTransport subscriptionInfoTransport = new MsmqTransport();
        private MsmqTransport messageBusTransport = new MsmqTransport();
        private MsmqSubscriberManager subscriberManager;
        private static ILog logger = LogManager.GetLogger(typeof(Distributor));

        #endregion
    }
}
